// General guidelines:
// - very unlikely you want to edit something else than _common.scss
// - keep the number of defined colors to a minimum, use the color blending functions if
//   you need a subtle shade
// - if you need to inverse a color function use the @if directive to match for dark $variant

$variant: 'dark';

@import '../Adwaita/colors';
@import 'colors-hc';
@import '../Adwaita/colors-public'; // export public colors
@import '../Adwaita/drawing';
@import '../Adwaita/common';
